"""PDFMagic (OpenWhisk action): convert a JPEG image to PDF.

Built as an Apache OpenWhisk action (i.e., serverless cloud function).
The image is received as a pair of bucket and object names, and fetched from
Google Cloud Storage using an API key provided in the parameters.
Then, the converted result is written back under a unique filename in the bucket
specified in the parameters.

Requirements:
* native dependencies:
  * imagemagick (command `convert`)
* Python dependencies:
  * delegator (package delegate.py)
  * Google Cloud SDK for storage and authentication (packages
        google-cloud-storage and google-auth)
"""

from uuid import uuid4 as uuid

import delegator
from google.cloud import storage as gcs
from google.oauth2 import service_account as gc_srvaccount

# The input image is written to this file.
IN_FILENAME = 'image.jpg'
# The output PDF is written to this file.
OUT_FILENAME = 'image.pdf'

ARGS = ('bucket', 'name', 'out-bucket', 'gcs-key')


def main(args):
    """Entrypoint of the action.

    Expected arguments:
    * bucket: the input bucket
    * name: the input filename in the bucket
    * out-bucket: the output bucket
    * gcs-key: Google Cloud Storage access key, a dict as produced by:
          gcloud iam service-accounts keys create key.json \
              --iam-account=SERVICE_ACCOUNT

    Return payload:
    * out-bucket: same value as the input parameter of the same name
    * name: the output filename in the output bucket

    Error return payload:
    * error: message describing the error
    """
    if any(expected_arg not in args for expected_arg in ARGS):
        return {
            'error': f'missing one or more parameters in {ARGS}',
        }

    gcs_creds = gc_srvaccount.Credentials.from_service_account_info(args['gcs-key'])
    gcs_creds = gcs_creds.with_scopes(
        ['https://www.googleapis.com/auth/cloud-platform'])

    # Client to Google Cloud Storage.
    gcs_client = gcs.Client(credentials=gcs_creds)
    # Name of the blob to store the converted file.
    out_blobname = f'pdf/{uuid()}.pdf'

    ### STUDENT CODE BELOW ###
    # 1. get a reference to the input bucket
    # 2. get the input file and write it to a local file named by the variable
    #    IN_FILENAME

    ### STUDENT CODE ABOVE ###

    # Call conversion command, and wait for completion.
    delegator.run(f'/usr/bin/convert {IN_FILENAME} {OUT_FILENAME}',
                                 block=True)

    ### STUDENT CODE BELOW ###
    # 1. get a reference to the output bucket
    # 2. write the converted output file named by the variable OUT_FILENAME, to
    #    the output blob named by the variable out_blobname.

    ### STUDENT CODE ABOVE ###

    ### STUDENT CODE BELOW ###
    # Return a dict conforming to the function docstring above.

    ### STUDENT CODE ABOVE ###
